@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2021] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2022] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -36,23 +36,29 @@
 # \  / _  __|_. _ _ |_)
 #  \/ (/_|  | |(_(_|| \/
 #                     /
-# VerticaPy is a Python library with scikit-like functionality to use to conduct
+# VerticaPy is a Python library with scikit-like functionality for conducting
 # data science projects on data stored in Vertica, taking advantage Vertica’s
 # speed and built-in analytics and machine learning features. It supports the
 # entire data science life cycle, uses a ‘pipeline’ mechanism to sequentialize
 # data transformation operations, and offers beautiful graphical options.
 #
-# VerticaPy aims to solve all of these problems. The idea is simple: instead
-# of moving data around for processing, VerticaPy brings the logic to the data.
+# VerticaPy aims to do all of the above. The idea is simple: instead of moving
+# data around for processing, VerticaPy brings the logic to the data.
 #
 #
-__version__ = "0.7.0"
+__version__ = "0.9.0"
 __author__ = "Badr Ouali"
 __author_email__ = "badr.ouali@vertica.com"
-__description__ = """VerticaPy simplifies data exploration, data cleaning and machine learning in Vertica."""
+__description__ = (
+    "VerticaPy simplifies data exploration, data cleaning"
+    " and machine learning in Vertica."
+)
 __url__ = "https://github.com/vertica/verticapy/"
 __license__ = "Apache License, Version 2.0"
 
+# Logo
+from verticapy.logo import *
+
 # vDataFrame
 from verticapy.vdataframe import *
 
@@ -68,27 +74,21 @@ import verticapy.stats
 # Learn
 import verticapy.learn
 
-try:
-    import tqdm
-
-    tqdm = True
-except:
-    tqdm = False
-
 verticapy.options = {
     "cache": True,
-    "max_rows": 100,
-    "max_columns": 50,
-    "percent_bar": None,
-    "print_info": True,
-    "query_on": False,
-    "time_on": False,
-    "mode": None,
-    "random_state": None,
     "colors": [],
     "color_style": "default",
-    "tqdm": tqdm,
-    "cursor": None,
-    "conn": None,
+    "connection": {"conn": None, "section": None, "dsn": None},
+    "max_columns": 50,
+    "max_rows": 100,
+    "mode": None,
+    "overwrite_model": True,
+    "percent_bar": None,
+    "print_info": True,
+    "sql_on": False,
+    "random_state": None,
+    "temp_schema": "public",
+    "time_on": False,
+    "tqdm": True,
     "vertica_version": None,
 }
