@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2022] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -36,14 +36,14 @@
 # \  / _  __|_. _ _ |_)
 #  \/ (/_|  | |(_(_|| \/
 #                     /
-# VerticaPy is a Python library with scikit-like functionality to use to conduct
+# VerticaPy is a Python library with scikit-like functionality for conducting
 # data science projects on data stored in Vertica, taking advantage Vertica’s
 # speed and built-in analytics and machine learning features. It supports the
 # entire data science life cycle, uses a ‘pipeline’ mechanism to sequentialize
 # data transformation operations, and offers beautiful graphical options.
 #
-# VerticaPy aims to solve all of these problems. The idea is simple: instead
-# of moving data around for processing, VerticaPy brings the logic to the data.
+# VerticaPy aims to do all of the above. The idea is simple: instead of moving
+# data around for processing, VerticaPy brings the logic to the data.
 #
 #
 # Modules
@@ -59,45 +59,44 @@ from verticapy.learn.vmodel import *
 class ElasticNet(Regressor):
     """
 ---------------------------------------------------------------------------
-Creates a ElasticNet object by using the Vertica Highly Distributed and 
-Scalable Linear Regression on the data. The Elastic Net is a regularized 
-regression method that linearly combines the L1 and L2 penalties of the 
-Lasso and Ridge methods. 
+Creates a ElasticNet object using the Vertica Linear Regression algorithm 
+on the data. The Elastic Net is a regularized regression method that 
+linearly combines the L1 and L2 penalties of the Lasso and Ridge methods.
 
 Parameters
 ----------
 name: str
 	Name of the the model. The model will be stored in the DB.
-cursor: DBcursor, optional
-	Vertica DB cursor.
 tol: float, optional
-	Determines whether the algorithm has reached the specified accuracy result.
+	Determines whether the algorithm has reached the specified accuracy 
+    result.
 C: float, optional
-	The regularization parameter value. The value must be zero or non-negative.
+	The regularization parameter value. The value must be zero or 
+    non-negative.
 max_iter: int, optional
-	Determines the maximum number of iterations the algorithm performs before 
-	achieving the specified accuracy result.
+	Determines the maximum number of iterations the algorithm performs 
+    before achieving the specified accuracy result.
 solver: str, optional
 	The optimizer method to use to train the model. 
 		Newton : Newton Method
 		BFGS   : Broyden Fletcher Goldfarb Shanno
 		CGD    : Coordinate Gradient Descent
 l1_ratio: float, optional
-	ENet mixture parameter that defines how much L1 versus L2 regularization 
-	to provide.
+	ENet mixture parameter that defines how much L1 versus L2 
+    regularization to provide.
 	"""
 
     def __init__(
         self,
         name: str,
-        cursor=None,
         tol: float = 1e-6,
         C: float = 1.0,
         max_iter: int = 100,
         solver: str = "CGD",
         l1_ratio: float = 0.5,
     ):
-        check_types([("name", name, [str],)])
+        version(condition=[8, 0, 0])
+        check_types([("name", name, [str])])
         self.type, self.name = "LinearRegression", name
         self.set_params(
             {
@@ -109,32 +108,28 @@ l1_ratio: float, optional
                 "l1_ratio": l1_ratio,
             }
         )
-        cursor = check_cursor(cursor)[0]
-        self.cursor = cursor
-        version(cursor=cursor, condition=[8, 0, 0])
 
 
 # ---#
 class Lasso(Regressor):
     """
 ---------------------------------------------------------------------------
-Creates a Lasso object by using the Vertica Highly Distributed and Scalable 
-Linear Regression on the data. The Lasso is a regularized regression method 
-which uses L1 penalty. 
+Creates a Lasso object using the Vertica Linear Regression algorithm on the 
+data. The Lasso is a regularized regression method which uses an L1 penalty.
 
 Parameters
 ----------
 name: str
 	Name of the the model. The model will be stored in the DB.
-cursor: DBcursor, optional
-	Vertica DB cursor.
 tol: float, optional
-	Determines whether the algorithm has reached the specified accuracy result.
+	Determines whether the algorithm has reached the specified accuracy 
+    result.
 C: float, optional
-    The regularization parameter value. The value must be zero or non-negative.
+    The regularization parameter value. The value must be zero or 
+    non-negative.
 max_iter: int, optional
-	Determines the maximum number of iterations the algorithm performs before 
-	achieving the specified accuracy result.
+	Determines the maximum number of iterations the algorithm performs 
+    before achieving the specified accuracy result.
 solver: str, optional
 	The optimizer method to use to train the model. 
 		Newton : Newton Method
@@ -145,13 +140,13 @@ solver: str, optional
     def __init__(
         self,
         name: str,
-        cursor=None,
         tol: float = 1e-6,
         C: float = 1.0,
         max_iter: int = 100,
         solver: str = "CGD",
     ):
-        check_types([("name", name, [str],)])
+        version(condition=[8, 0, 0])
+        check_types([("name", name, [str])])
         self.type, self.name = "LinearRegression", name
         self.set_params(
             {
@@ -163,30 +158,27 @@ solver: str, optional
             }
         )
         for elem in ["l1_ratio"]:
-            del self.parameters[elem]
-        cursor = check_cursor(cursor)[0]
-        self.cursor = cursor
-        version(cursor=cursor, condition=[8, 0, 0])
+            if elem in self.parameters:
+                del self.parameters[elem]
 
 
 # ---#
 class LinearRegression(Regressor):
     """
 ---------------------------------------------------------------------------
-Creates a LinearRegression object by using the Vertica Highly Distributed and 
-Scalable Linear Regression on the data. 
+Creates a LinearRegression object using the Vertica Linear Regression 
+algorithm on the data.
 
 Parameters
 ----------
 name: str
 	Name of the the model. The model will be stored in the DB.
-cursor: DBcursor, optional
-	Vertica DB cursor.
 tol: float, optional
-	Determines whether the algorithm has reached the specified accuracy result.
+	Determines whether the algorithm has reached the specified accuracy 
+    result.
 max_iter: int, optional
-	Determines the maximum number of iterations the algorithm performs before 
-	achieving the specified accuracy result.
+	Determines the maximum number of iterations the algorithm performs 
+    before achieving the specified accuracy result.
 solver: str, optional
 	The optimizer method to use to train the model. 
 		Newton : Newton Method
@@ -194,15 +186,11 @@ solver: str, optional
 	"""
 
     def __init__(
-        self,
-        name: str,
-        cursor=None,
-        tol: float = 1e-6,
-        max_iter: int = 100,
-        solver: str = "Newton",
+        self, name: str, tol: float = 1e-6, max_iter: int = 100, solver: str = "Newton"
     ):
+        version(condition=[8, 0, 0])
         check_types(
-            [("name", name, [str],), ("solver", solver.lower(), ["newton", "bfgs"],),]
+            [("name", name, [str]), ("solver", solver.lower(), ["newton", "bfgs"])]
         )
         self.type, self.name = "LinearRegression", name
         self.set_params(
@@ -214,25 +202,21 @@ solver: str, optional
             }
         )
         for elem in ["l1_ratio", "C"]:
-            del self.parameters[elem]
-        cursor = check_cursor(cursor)[0]
-        self.cursor = cursor
-        version(cursor=cursor, condition=[8, 0, 0])
+            if elem in self.parameters:
+                del self.parameters[elem]
 
 
 # ---#
 class LogisticRegression(BinaryClassifier):
     """
 ---------------------------------------------------------------------------
-Creates a LogisticRegression object by using the Vertica Highly Distributed 
-and Scalable Logistic Regression on the data.
+Creates a LogisticRegression object using the Vertica Logistic Regression
+algorithm on the data.
 
 Parameters
 ----------
 name: str
 	Name of the the model. The model will be stored in the DB.
-cursor: DBcursor, optional
-	Vertica DB cursor.
 penalty: str, optional
 	Determines the method of regularization.
 		None : No Regularization
@@ -259,7 +243,6 @@ l1_ratio: float, optional
     def __init__(
         self,
         name: str,
-        cursor=None,
         penalty: str = "None",
         tol: float = 1e-6,
         C: int = 1,
@@ -267,7 +250,8 @@ l1_ratio: float, optional
         solver: str = "Newton",
         l1_ratio: float = 0.5,
     ):
-        check_types([("name", name, [str],)])
+        version(condition=[8, 0, 0])
+        check_types([("name", name, [str])])
         self.type, self.name = "LogisticRegression", name
         self.set_params(
             {
@@ -281,38 +265,36 @@ l1_ratio: float, optional
         )
         if penalty.lower() == "none":
             for elem in ["l1_ratio", "C"]:
-                del self.parameters[elem]
-            check_types([("solver", solver.lower(), ["bfgs", "newton"],)])
-        elif penalty.lower() == "l2":
+                if elem in self.parameters:
+                    del self.parameters[elem]
+            check_types([("solver", solver.lower(), ["bfgs", "newton"])])
+        elif penalty.lower() in ("l1", "l2"):
             for elem in ["l1_ratio"]:
-                del self.parameters[elem]
-            check_types([("solver", solver.lower(), ["bfgs", "newton"],)])
-        cursor = check_cursor(cursor)[0]
-        self.cursor = cursor
-        version(cursor=cursor, condition=[8, 0, 0])
+                if elem in self.parameters:
+                    del self.parameters[elem]
+            check_types([("solver", solver.lower(), ["bfgs", "newton", "cgd"])])
 
 
 # ---#
 class Ridge(Regressor):
     """
 ---------------------------------------------------------------------------
-Creates a Ridge object by using the Vertica Highly Distributed and Scalable 
-Linear Regression on the data. The Ridge is a regularized regression method 
-which uses L2 penalty. 
+Creates a Ridge object using the Vertica Linear Regression algorithm on the 
+data. The Ridge is a regularized regression method which uses an L2 penalty. 
 
 Parameters
 ----------
 name: str
 	Name of the the model. The model will be stored in the DB.
-cursor: DBcursor, optional
-	Vertica DB cursor.
 tol: float, optional
-	Determines whether the algorithm has reached the specified accuracy result.
+	Determines whether the algorithm has reached the specified 
+    accuracy result.
 C: float, optional
-    The regularization parameter value. The value must be zero or non-negative.
+    The regularization parameter value. The value must be zero 
+    or non-negative.
 max_iter: int, optional
-	Determines the maximum number of iterations the algorithm performs before 
-	achieving the specified accuracy result.
+	Determines the maximum number of iterations the algorithm 
+    performs before achieving the specified accuracy result.
 solver: str, optional
 	The optimizer method to use to train the model. 
 		Newton : Newton Method
@@ -322,14 +304,14 @@ solver: str, optional
     def __init__(
         self,
         name: str,
-        cursor=None,
         tol: float = 1e-6,
         C: float = 1.0,
         max_iter: int = 100,
         solver: str = "Newton",
     ):
+        version(condition=[8, 0, 0])
         check_types(
-            [("name", name, [str], ("solver", solver.lower(), ["newton", "bfgs"],),)]
+            [("name", name, [str], ("solver", solver.lower(), ["newton", "bfgs"]))]
         )
         self.type, self.name = "LinearRegression", name
         self.set_params(
@@ -342,7 +324,5 @@ solver: str, optional
             }
         )
         for elem in ["l1_ratio"]:
-            del self.parameters[elem]
-        cursor = check_cursor(cursor)[0]
-        self.cursor = cursor
-        version(cursor=cursor, condition=[8, 0, 0])
+            if elem in self.parameters:
+                del self.parameters[elem]
