@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2022] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -11,50 +11,36 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pytest, warnings
-from verticapy import vDataFrame
-from verticapy import drop_table
+# Pytest
+import pytest
 
-from verticapy import set_option
+# VerticaPy
+from verticapy import drop, set_option
+from verticapy.datasets import load_smart_meters, load_titanic, load_amazon
 
 set_option("print_info", False)
 set_option("random_state", 0)
 
 
 @pytest.fixture(scope="module")
-def smart_meters_vd(base):
-    from verticapy.learn.datasets import load_smart_meters
-
-    smart_meters = load_smart_meters(cursor=base.cursor)
+def smart_meters_vd():
+    smart_meters = load_smart_meters()
     yield smart_meters
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(
-            name="public.smart_meters", cursor=base.cursor,
-        )
+    drop(name="public.smart_meters")
 
 
 @pytest.fixture(scope="module")
-def titanic_vd(base):
-    from verticapy.learn.datasets import load_titanic
-
-    titanic = load_titanic(cursor=base.cursor)
+def titanic_vd():
+    titanic = load_titanic()
     yield titanic
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(
-            name="public.titanic", cursor=base.cursor,
-        )
+    drop(name="public.titanic")
 
 
 @pytest.fixture(scope="module")
-def amazon_vd(base):
-    from verticapy.learn.datasets import load_amazon
-
-    amazon = load_amazon(cursor=base.cursor)
+def amazon_vd():
+    amazon = load_amazon()
     yield amazon
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(
-            name="public.amazon", cursor=base.cursor,
-        )
+    drop(name="public.amazon")
 
 
 class TestvDFFilterSample:
@@ -92,9 +78,28 @@ class TestvDFFilterSample:
         assert result2["pclass"][1] == 1
 
     def test_vDF_at_time(self, smart_meters_vd):
-        result = smart_meters_vd.copy().at_time(ts="time", time="12:00",)
+        result = smart_meters_vd.copy().at_time(ts="time", time="12:00")
         assert result.shape() == (140, 3)
 
+    def test_vDF_balance(self, titanic_vd):
+        # hybrid
+        result = titanic_vd.balance("embarked", method="hybrid")["embarked"].topk()
+        assert 30 < result["percent"][0] < 40
+        assert 30 < result["percent"][1] < 40
+        assert 30 < result["percent"][2] < 40
+        # under
+        result = titanic_vd.balance("embarked", method="under", x=0.5)[
+            "embarked"
+        ].topk()
+        assert 35 < result["percent"][0] < 55
+        assert 35 < result["percent"][1] < 55
+        assert 15 < result["percent"][2] < 30
+        # over
+        result = titanic_vd.balance("embarked", method="over", x=0.5)["embarked"].topk()
+        assert 40 < result["percent"][0] < 55
+        assert 15 < result["percent"][1] < 35
+        assert 15 < result["percent"][2] < 35
+
     def test_vDF_between_time(self, smart_meters_vd):
         result = smart_meters_vd.copy().between_time(
             ts="time", start_time="12:00", end_time="14:00",
@@ -103,13 +108,12 @@ class TestvDFFilterSample:
 
     def test_vDF_filter(self, titanic_vd):
         result = titanic_vd.copy().filter(
-            expr="pclass = 1 OR age > 50",
-            conditions=["embarked = 'S'", "boat IS NOT NULL"],
+            ["pclass = 1 OR age > 50", "embarked = 'S'", "boat IS NOT NULL"],
         )
-        assert result.shape() == (343, 14)
+        assert result.shape() == (99, 14)
 
     def test_vDF_first(self, smart_meters_vd):
-        result = smart_meters_vd.copy().first(ts="time", offset="6 months",)
+        result = smart_meters_vd.copy().first(ts="time", offset="6 months")
         assert result.shape() == (3427, 3)
 
     def test_vDF_isin(self, amazon_vd):
@@ -124,7 +128,7 @@ class TestvDFFilterSample:
         ).shape() == (478, 3)
 
     def test_vDF_last(self, smart_meters_vd):
-        result = smart_meters_vd.copy().last(ts="time", offset="1 year",)
+        result = smart_meters_vd.copy().last(ts="time", offset="1 year")
         assert result.shape() == (7018, 3)
 
     def test_vDF_drop(self, titanic_vd):
@@ -137,12 +141,12 @@ class TestvDFFilterSample:
         assert result.shape() == (1234, 13)
 
     def test_vDF_drop_duplicates(self, titanic_vd):
-        result = titanic_vd.copy().drop_duplicates(columns=["age", "fare", "pclass"],)
+        result = titanic_vd.copy().drop_duplicates(columns=["age", "fare", "pclass"])
         assert result.shape() == (942, 14)
 
     def test_vDF_drop_outliers(self, titanic_vd):
         # testing with threshold
-        result1 = titanic_vd.copy()["age"].drop_outliers(threshold=3.0,)
+        result1 = titanic_vd.copy()["age"].drop_outliers(threshold=3.0)
         assert result1.shape() == (994, 14)
 
         # testing without threshold
@@ -164,7 +168,7 @@ class TestvDFFilterSample:
         result = titanic_vd.copy().sample(x=0.33, method="random")
         assert result.shape()[0] == pytest.approx(1234 * 0.33, 0.12)
         result2 = titanic_vd.copy().sample(
-            x=0.33, method="stratified", by=["age", "pclass",]
+            x=0.33, method="stratified", by=["age", "pclass"]
         )
         assert result2.shape()[0] == pytest.approx(1234 * 0.33, 0.12)
         result3 = titanic_vd.copy().sample(x=0.33, method="systematic")
@@ -174,7 +178,7 @@ class TestvDFFilterSample:
         result = titanic_vd.copy().sample(n=200, method="random")
         assert result.shape()[0] == pytest.approx(200, 0.12)
         result2 = titanic_vd.copy().sample(
-            n=200, method="stratified", by=["age", "pclass",]
+            n=200, method="stratified", by=["age", "pclass"]
         )
         assert result2.shape()[0] == pytest.approx(200, 0.12)
         result3 = titanic_vd.copy().sample(n=200, method="systematic")
