@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2022] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -11,52 +11,80 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pytest, os, warnings
+# Pytest
+import pytest
+
+# Standard Python Modules
+import os, pickle
 from math import ceil, floor
-from verticapy import vDataFrame, get_session, read_vdf, drop_table, drop_view
-from verticapy import set_option
+
+# Other Modules
+import pandas, geopandas
+
+# VerticaPy
+from verticapy import (
+    vDataFrame,
+    get_session,
+    drop,
+    set_option,
+    read_shp,
+)
+from verticapy.connect import current_cursor
 import verticapy.stats as st
+from verticapy.datasets import load_titanic, load_cities, load_amazon, load_world
 
 set_option("print_info", False)
 
 
 @pytest.fixture(scope="module")
-def titanic_vd(base):
-    from verticapy.learn.datasets import load_titanic
-
-    titanic = load_titanic(cursor=base.cursor)
+def titanic_vd():
+    titanic = load_titanic()
     yield titanic
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(
-            name="public.titanic", cursor=base.cursor,
-        )
+    drop(name="public.titanic")
 
 
 @pytest.fixture(scope="module")
-def amazon_vd(base):
-    from verticapy.learn.datasets import load_amazon
+def cities_vd():
+    cities = load_cities()
+    yield cities
+    drop(name="public.cities")
 
-    amazon = load_amazon(cursor=base.cursor)
+
+@pytest.fixture(scope="module")
+def amazon_vd():
+    amazon = load_amazon()
     yield amazon
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(
-            name="public.amazon", cursor=base.cursor,
-        )
+    drop(name="public.amazon")
 
 
 @pytest.fixture(scope="module")
-def world_vd(base):
-    from verticapy.learn.datasets import load_world
-
-    world = load_world(cursor=base.cursor)
+def world_vd():
+    world = load_world()
     yield world
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(
-            name="public.world", cursor=base.cursor,
-        )
+    drop(name="public.world")
 
 
 class TestvDFUtilities:
+    def test_vDF_repr(self, titanic_vd):
+        # vdf
+        repr_vdf = titanic_vd.__repr__()
+        assert "pclass" in repr_vdf
+        assert "survived" in repr_vdf
+        assert 10000 < len(repr_vdf) < 1000000
+        repr_html_vdf = titanic_vd._repr_html_()
+        assert 10000 < len(repr_html_vdf) < 10000000
+        assert "<table>" in repr_html_vdf
+        assert "data:image/png;base64," in repr_html_vdf
+        # vdc
+        repr_vdc = titanic_vd["age"].__repr__()
+        assert "age" in repr_vdc
+        assert "60" in repr_vdc
+        assert 500 < len(repr_vdc) < 5000
+        repr_html_vdc = titanic_vd["age"]._repr_html_()
+        assert 10000 < len(repr_html_vdc) < 10000000
+        assert "<table>" in repr_html_vdc
+        assert "data:image/png;base64," in repr_html_vdc
+
     def test_vDF_magic(self, titanic_vd):
         assert (
             str(titanic_vd["name"]._in(["Madison", "Ashley", None]))
@@ -116,34 +144,38 @@ class TestvDFUtilities:
         assert str(3 == titanic_vd["age"]) == '("age") = (3)'
         assert str(titanic_vd["age"] != 3) == '("age") != (3)'
         assert str(None != titanic_vd["age"]) == '("age") IS NOT NULL'
+        assert titanic_vd["fare"][0] >= 0
+        assert titanic_vd[["fare"]][0][0] >= 0
+        assert titanic_vd[titanic_vd["fare"] > 500].shape()[0] == 4
+        assert titanic_vd[titanic_vd["fare"] < 500].shape()[0] == 1229
+        assert titanic_vd[titanic_vd["fare"] * 4 + 2 < 500].shape()[0] == 1167
+        assert titanic_vd[titanic_vd["fare"] / 4 - 2 < 500].shape()[0] == 1233
 
     def test_vDF_to_csv(self, titanic_vd):
-        session_id = get_session(titanic_vd._VERTICAPY_VARIABLES_["cursor"])
         titanic_vd.copy().select(["age", "fare"]).sort({"age": "desc", "fare": "desc"})[
             0:2
-        ].to_csv("verticapy_test_{}".format(session_id))
+        ].to_csv("verticapy_test_to_csv.csv")
         try:
-            file = open("verticapy_test_{}.csv".format(session_id), "r")
+            file = open("verticapy_test_to_csv.csv", "r")
             result = file.read()
             assert result == "age,fare\n80.000,30.00000\n76.000,78.85000"
         except:
-            os.remove("verticapy_test_{}.csv".format(session_id))
+            os.remove("verticapy_test_to_csv.csv")
             file.close()
             raise
-        os.remove("verticapy_test_{}.csv".format(session_id))
+        os.remove("verticapy_test_to_csv.csv")
         file.close()
+        # TODO - test with multiple CSV files.
+
+    def test_vDF_to_parquet(self, titanic_vd):
+        session_id = get_session()
+        name = "parquet_test_{}".format(session_id)
+        result = titanic_vd.to_parquet(name)
+        assert result["Rows Exported"][0] == 1234
+        # TODO: erasing the folder
 
     def test_vDF_to_db(self, titanic_vd):
-        try:
-            with warnings.catch_warnings(record=True) as w:
-                drop_view(
-                    "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-                )
-                drop_table(
-                    "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-                )
-        except:
-            pass
+        drop("verticapy_titanic_tmp")
         # testing relation_type = view
         try:
             titanic_vd.copy().to_db(
@@ -153,28 +185,19 @@ class TestvDFUtilities:
                 db_filter="age > 40",
                 nb_split=3,
             )
-            titanic_tmp = vDataFrame(
-                "verticapy_titanic_tmp",
-                cursor=titanic_vd._VERTICAPY_VARIABLES_["cursor"],
-            )
+            titanic_tmp = vDataFrame("verticapy_titanic_tmp")
             assert titanic_tmp.shape() == (220, 4)
             assert titanic_tmp["_verticapy_split_"].min() == 0
             assert titanic_tmp["_verticapy_split_"].max() == 2
-            titanic_vd._VERTICAPY_VARIABLES_["cursor"].execute(
+            current_cursor().execute(
                 "SELECT table_name FROM view_columns WHERE table_name = 'verticapy_titanic_tmp'"
             )
-            result = titanic_vd._VERTICAPY_VARIABLES_["cursor"].fetchone()
+            result = current_cursor().fetchone()
             assert result[0] == "verticapy_titanic_tmp"
         except:
-            with warnings.catch_warnings(record=True) as w:
-                drop_view(
-                    "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-                )
+            drop("verticapy_titanic_tmp", method="view")
             raise
-        with warnings.catch_warnings(record=True) as w:
-            drop_view(
-                "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-            )
+        drop("verticapy_titanic_tmp", method="view")
         # testing relation_type = table
         try:
             titanic_vd.copy().to_db(
@@ -184,28 +207,19 @@ class TestvDFUtilities:
                 db_filter="age > 40",
                 nb_split=3,
             )
-            titanic_tmp = vDataFrame(
-                "verticapy_titanic_tmp",
-                cursor=titanic_vd._VERTICAPY_VARIABLES_["cursor"],
-            )
+            titanic_tmp = vDataFrame("verticapy_titanic_tmp")
             assert titanic_tmp.shape() == (220, 4)
             assert titanic_tmp["_verticapy_split_"].min() == 0
             assert titanic_tmp["_verticapy_split_"].max() == 2
-            titanic_vd._VERTICAPY_VARIABLES_["cursor"].execute(
+            current_cursor().execute(
                 "SELECT table_name FROM columns WHERE table_name = 'verticapy_titanic_tmp'"
             )
-            result = titanic_vd._VERTICAPY_VARIABLES_["cursor"].fetchone()
+            result = current_cursor().fetchone()
             assert result[0] == "verticapy_titanic_tmp"
         except:
-            with warnings.catch_warnings(record=True) as w:
-                drop_table(
-                    "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-                )
+            drop("verticapy_titanic_tmp")
             raise
-        with warnings.catch_warnings(record=True) as w:
-            drop_table(
-                "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-            )
+        drop("verticapy_titanic_tmp")
         # testing relation_type = temporary table
         try:
             titanic_vd.copy().to_db(
@@ -215,28 +229,19 @@ class TestvDFUtilities:
                 db_filter="age > 40",
                 nb_split=3,
             )
-            titanic_tmp = vDataFrame(
-                "verticapy_titanic_tmp",
-                cursor=titanic_vd._VERTICAPY_VARIABLES_["cursor"],
-            )
+            titanic_tmp = vDataFrame("verticapy_titanic_tmp")
             assert titanic_tmp.shape() == (220, 4)
             assert titanic_tmp["_verticapy_split_"].min() == 0
             assert titanic_tmp["_verticapy_split_"].max() == 2
-            titanic_vd._VERTICAPY_VARIABLES_["cursor"].execute(
+            current_cursor().execute(
                 "SELECT table_name FROM columns WHERE table_name = 'verticapy_titanic_tmp'"
             )
-            result = titanic_vd._VERTICAPY_VARIABLES_["cursor"].fetchone()
+            result = current_cursor().fetchone()
             assert result[0] == "verticapy_titanic_tmp"
         except:
-            with warnings.catch_warnings(record=True) as w:
-                drop_table(
-                    "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-                )
+            drop("verticapy_titanic_tmp")
             raise
-        with warnings.catch_warnings(record=True) as w:
-            drop_table(
-                "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-            )
+        drop("verticapy_titanic_tmp")
         # testing relation_type = temporary local table
         try:
             titanic_vd.copy().to_db(
@@ -246,48 +251,39 @@ class TestvDFUtilities:
                 db_filter="age > 40",
                 nb_split=3,
             )
-            titanic_tmp = vDataFrame(
-                "v_temp_schema.verticapy_titanic_tmp",
-                cursor=titanic_vd._VERTICAPY_VARIABLES_["cursor"],
-            )
+            titanic_tmp = vDataFrame("v_temp_schema.verticapy_titanic_tmp")
             assert titanic_tmp.shape() == (220, 4)
             assert titanic_tmp["_verticapy_split_"].min() == 0
             assert titanic_tmp["_verticapy_split_"].max() == 2
-            titanic_vd._VERTICAPY_VARIABLES_["cursor"].execute(
+            current_cursor().execute(
                 "SELECT table_name FROM columns WHERE table_name = 'verticapy_titanic_tmp'"
             )
-            result = titanic_vd._VERTICAPY_VARIABLES_["cursor"].fetchone()
+            result = current_cursor().fetchone()
             assert result[0] == "verticapy_titanic_tmp"
         except:
-            with warnings.catch_warnings(record=True) as w:
-                drop_table(
-                    "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-                )
+            drop("verticapy_titanic_tmp")
             raise
-        with warnings.catch_warnings(record=True) as w:
-            drop_table(
-                "verticapy_titanic_tmp", titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-            )
+        drop("verticapy_titanic_tmp")
 
     def test_vDF_to_json(self, titanic_vd):
-        session_id = get_session(titanic_vd._VERTICAPY_VARIABLES_["cursor"])
+        session_id = get_session()
         titanic_vd.copy().select(["age", "fare"]).sort({"age": "desc", "fare": "desc"})[
             0:2
-        ].to_json("verticapy_test_{}".format(session_id))
+        ].to_json("verticapy_test_to_json.json")
         try:
-            file = open("verticapy_test_{}.json".format(session_id), "r")
+            file = open("verticapy_test_to_json.json", "r")
             result = file.read()
-            print(result)
             assert (
                 result
                 == '[\n{"age": 80.000, "fare": 30.00000},\n{"age": 76.000, "fare": 78.85000},\n]'
             )
         except:
-            os.remove("verticapy_test_{}.json".format(session_id))
+            os.remove("verticapy_test_to_json.json")
             file.close()
             raise
-        os.remove("verticapy_test_{}.json".format(session_id))
+        os.remove("verticapy_test_to_json.json")
         file.close()
+        # TODO - test with multiple JSON files.
 
     def test_vDF_to_list(self, titanic_vd):
         result = titanic_vd.select(["age", "survived"])[:20].to_list()
@@ -296,35 +292,37 @@ class TestvDFUtilities:
 
     def test_vDF_to_numpy(self, titanic_vd):
         result = titanic_vd.select(["age", "survived"])[:20].to_numpy()
-        result.shape == (20, 2)
+        assert result.shape == (20, 2)
 
     def test_vDF_to_pandas(self, titanic_vd):
-        import pandas
-
         result = titanic_vd.to_pandas()
         assert isinstance(result, pandas.DataFrame)
         assert result.shape == (1234, 14)
 
-    def test_vDF_to_geopandas(self, world_vd):
-        import geopandas
+    def test_vDF_to_pickle(self, titanic_vd):
+        result = titanic_vd.select(["age", "survived"])[:20].to_pickle("save.p")
+        pickle.DEFAULT_PROTOCOL = 4
+        result_tmp = pickle.load(open("save.p", "rb"))
+        assert result_tmp.shape() == (20, 2)
+        os.remove("save.p")
 
+    def test_vDF_to_geopandas(self, world_vd):
         result = world_vd.to_geopandas(geometry="geometry")
         assert isinstance(result, geopandas.GeoDataFrame)
         assert result.shape == (177, 4)
 
-    def test_vDF_to_vdf(self, titanic_vd):
-        session_id = get_session(titanic_vd._VERTICAPY_VARIABLES_["cursor"])
-        titanic_vd.to_vdf("verticapy_test_{}".format(session_id))
+    def test_vDF_to_shp(self, cities_vd):
+        drop(name="public.cities_test")
+        cities_vd.to_shp("cities_test", "/home/dbadmin/", shape="Point")
+        vdf = read_shp("/home/dbadmin/cities_test.shp")
+        assert vdf.shape() == (202, 3)
         try:
-            result = read_vdf(
-                "verticapy_test_{}.vdf".format(session_id),
-                cursor=titanic_vd._VERTICAPY_VARIABLES_["cursor"],
-            )
+            os.remove("/home/dbadmin/cities_test.shp")
+            os.remove("/home/dbadmin/cities_test.shx")
+            os.remove("/home/dbadmin/cities_test.dbf")
         except:
-            result = False
-        os.remove("verticapy_test_{}.vdf".format(session_id))
-        assert isinstance(result, vDataFrame)
-        assert result.shape() == (1234, 14)
+            pass
+        drop(name="public.cities_test")
 
     def test_vDF_del_catalog(self, titanic_vd):
         result = titanic_vd.copy()
@@ -353,17 +351,6 @@ class TestvDFUtilities:
         result.save()
         assert len(result._VERTICAPY_VARIABLES_["saving"]) == 1
 
-    def test_vDF_set_cursor(self, titanic_vd):
-        result = titanic_vd.copy()
-        cursor = titanic_vd._VERTICAPY_VARIABLES_["cursor"]
-        result.set_cursor(cursor)
-        assert isinstance(result._VERTICAPY_VARIABLES_["cursor"], type(cursor))
-
-    def test_vDF_set_schema_writing(self, titanic_vd):
-        result = titanic_vd.copy()
-        result.set_schema_writing("test")
-        assert result._VERTICAPY_VARIABLES_["schema_writing"] == "test"
-
     def test_vDF_catcol(self, titanic_vd):
         result = [
             elem.replace('"', "").lower()
@@ -525,18 +512,19 @@ class TestvDFUtilities:
         result = amazon_vd["state"].isnum()
         assert result == False
 
+    @pytest.mark.skip(reason="test not stable")
     def test_vDF_memory_usage(self, amazon_vd):
         # testing vDataFrame[].memory_usage
         result = amazon_vd["number"].memory_usage()
-        assert result == pytest.approx(1714, 5e-2)
+        assert result == pytest.approx(1690, 5e-2)
 
         # testing vDataFrame.memory_usage
         result2 = amazon_vd.memory_usage()
-        assert result2["value"][0] == pytest.approx(862, 5e-2)
-        assert result2["value"][1] == pytest.approx(1712, 5e-2)
-        assert result2["value"][2] == pytest.approx(1713, 5e-2)
-        assert result2["value"][3] == pytest.approx(1714, 5e-2)
-        assert result2["value"][4] == pytest.approx(6001, 5e-2)
+        assert result2["value"][0] == pytest.approx(684, 5e-2)
+        assert result2["value"][1] == pytest.approx(1688, 5e-2)
+        assert result2["value"][2] == pytest.approx(1689, 5e-2)
+        assert result2["value"][3] == pytest.approx(1690, 5e-2)
+        assert result2["value"][4] == pytest.approx(5751, 5e-2)
 
     def test_vDF_numcol(self, titanic_vd):
         result = [elem.replace('"', "") for elem in titanic_vd.numcol()]
@@ -553,6 +541,18 @@ class TestvDFUtilities:
         assert result["age"] == [76.0, 80.0]
         assert result["fare"] == [78.85, 30.0]
 
+    def test_vDF_sql(self, titanic_vd):
+        sql = """-- Selecting some columns \n
+                 SELECT 
+                    age, 
+                    fare 
+                 FROM titanic 
+                 WHERE age IS NOT NULL;;"""
+        vdf = vDataFrame(sql=sql)
+        assert vdf.shape() == (997, 2)
+        vdf = vDataFrame(sql=sql, usecols=["age"])
+        assert vdf.shape() == (997, 1)
+
     def test_vDF_store_usage(self, titanic_vd):
         result = titanic_vd["age"].store_usage()
         assert result == pytest.approx(5908, 1e-2)
