@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2022] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -11,30 +11,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pytest, warnings
-from verticapy.learn.linear_model import LinearRegression
-from verticapy import drop_table
+# Standard Python Modules
+import pytest
+
+# Other Modules
 import matplotlib.pyplot as plt
 
-from verticapy import set_option
+# VerticaPy
+from verticapy import drop, set_option
+from verticapy.connect import current_cursor
+from verticapy.datasets import load_winequality
+from verticapy.learn.linear_model import LinearRegression
 
 set_option("print_info", False)
 
 
 @pytest.fixture(scope="module")
-def winequality_vd(base):
-    from verticapy.learn.datasets import load_winequality
-
-    winequality = load_winequality(cursor=base.cursor)
+def winequality_vd():
+    winequality = load_winequality()
     yield winequality
-    with warnings.catch_warnings(record=True) as w:
-        drop_table(name="public.winequality", cursor=base.cursor)
+    drop(name="public.winequality",)
 
 
 @pytest.fixture(scope="module")
-def model(base, winequality_vd):
-    base.cursor.execute("DROP MODEL IF EXISTS linreg_model_test")
-    model_class = LinearRegression("linreg_model_test", cursor=base.cursor)
+def model(winequality_vd):
+    model_class = LinearRegression("linreg_model_test",)
+    model_class.drop()
     model_class.fit(
         "public.winequality", ["citric_acid", "residual_sugar", "alcohol"], "quality"
     )
@@ -42,28 +44,44 @@ def model(base, winequality_vd):
     model_class.drop()
 
 
-class TestLogisticRegression:
+class TestLinearRegression:
+    def test_repr(self, model):
+        assert "|coefficient|std_err |t_value |p_value" in model.__repr__()
+        model_repr = LinearRegression("lin_repr")
+        model_repr.drop()
+        assert model_repr.__repr__() == "<LinearRegression>"
+
+    def test_contour(self, winequality_vd):
+        model_test = LinearRegression("model_contour",)
+        model_test.drop()
+        model_test.fit(
+            winequality_vd, ["citric_acid", "residual_sugar"], "quality",
+        )
+        result = model_test.contour()
+        assert len(result.get_default_bbox_extra_artists()) == 32
+        model_test.drop()
+
     def test_deploySQL(self, model):
         expected_sql = 'PREDICT_LINEAR_REG("citric_acid", "residual_sugar", "alcohol" USING PARAMETERS model_name = \'linreg_model_test\', match_by_pos = \'true\')'
         result_sql = model.deploySQL()
 
         assert result_sql == expected_sql
 
-    def test_drop(self, base):
-        base.cursor.execute("DROP MODEL IF EXISTS linreg_model_test_drop")
-        model_test = LinearRegression("linreg_model_test_drop", cursor=base.cursor)
+    def test_drop(self):
+        current_cursor().execute("DROP MODEL IF EXISTS linreg_model_test_drop")
+        model_test = LinearRegression("linreg_model_test_drop",)
         model_test.fit("public.winequality", ["alcohol"], "quality")
 
-        base.cursor.execute(
+        current_cursor().execute(
             "SELECT model_name FROM models WHERE model_name = 'linreg_model_test_drop'"
         )
-        assert base.cursor.fetchone()[0] == "linreg_model_test_drop"
+        assert current_cursor().fetchone()[0] == "linreg_model_test_drop"
 
         model_test.drop()
-        base.cursor.execute(
+        current_cursor().execute(
             "SELECT model_name FROM models WHERE model_name = 'linreg_model_test_drop'"
         )
-        assert base.cursor.fetchone() is None
+        assert current_cursor().fetchone() is None
 
     def test_features_importance(self, model):
         fim = model.features_importance()
@@ -71,7 +89,7 @@ class TestLogisticRegression:
         assert fim["index"] == ["alcohol", "residual_sugar", "citric_acid"]
         assert fim["importance"] == [52.25, 32.58, 15.17]
         assert fim["sign"] == [1, 1, 1]
-        plt.close()
+        plt.close("all")
 
     def test_get_attr(self, model):
         m_att = model.get_attr()
@@ -131,23 +149,53 @@ class TestLogisticRegression:
             "tol": 1e-06,
         }
 
-    @pytest.mark.skip(reason="test not implemented")
-    def test_get_plot(self):
-        pass
+    def test_get_plot(self, winequality_vd):
+        current_cursor().execute("DROP MODEL IF EXISTS model_test_plot")
+        model_test = LinearRegression("model_test_plot",)
+        model_test.fit(winequality_vd, ["alcohol"], "quality")
+        result = model_test.plot(color="r")
+        assert len(result.get_default_bbox_extra_artists()) == 9
+        plt.close("all")
+        model_test.drop()
+        model_test.fit(winequality_vd, ["alcohol", "residual_sugar"], "quality")
+        result = model_test.plot(color="r")
+        assert len(result.get_default_bbox_extra_artists()) == 3
+        plt.close("all")
+        model_test.drop()
 
-    def test_to_sklearn(self, model):
-        md = model.to_sklearn()
-        model.cursor.execute(
+    def test_to_python(self, model):
+        current_cursor().execute(
             "SELECT PREDICT_LINEAR_REG(3.0, 11.0, 93. USING PARAMETERS model_name = '{}', match_by_pos=True)".format(
                 model.name
             )
         )
-        prediction = model.cursor.fetchone()[0]
-        assert prediction == pytest.approx(md.predict([[3.0, 11.0, 93.0]])[0][0])
+        prediction = current_cursor().fetchone()[0]
+        assert prediction == pytest.approx(
+            model.to_python(return_str=False)([[3.0, 11.0, 93.0]])[0]
+        )
 
-    def test_shapExplainer(self, model):
-        explainer = model.shapExplainer()
-        assert explainer.expected_value[0] == pytest.approx(5.81837771)
+    def test_to_sql(self, model):
+        current_cursor().execute(
+            "SELECT PREDICT_LINEAR_REG(3.0, 11.0, 93. USING PARAMETERS model_name = '{}', match_by_pos=True)::float, {}::float".format(
+                model.name, model.to_sql([3.0, 11.0, 93.0])
+            )
+        )
+        prediction = current_cursor().fetchone()
+        assert prediction[0] == pytest.approx(prediction[1])
+
+    def test_to_memmodel(self, model, winequality_vd):
+        mmodel = model.to_memmodel()
+        res = mmodel.predict([[3.0, 11.0, 93.0], [11.0, 1.0, 99.0]])
+        res_py = model.to_python()([[3.0, 11.0, 93.0], [11.0, 1.0, 99.0]])
+        assert res[0] == res_py[0]
+        assert res[1] == res_py[1]
+        vdf = winequality_vd.copy()
+        vdf["prediction_sql"] = mmodel.predict_sql(
+            ["citric_acid", "residual_sugar", "alcohol"]
+        )
+        model.predict(vdf, name="prediction_vertica_sql")
+        score = vdf.score("prediction_sql", "prediction_vertica_sql", "r2")
+        assert score == pytest.approx(1.0)
 
     def test_get_predicts(self, winequality_vd, model):
         winequality_copy = winequality_vd.copy()
@@ -162,7 +210,7 @@ class TestLogisticRegression:
         )
 
     def test_regression_report(self, model):
-        reg_rep = model.regression_report()
+        reg_rep = model.regression_report("metrics")
 
         assert reg_rep["index"] == [
             "explained_variance",
@@ -170,14 +218,46 @@ class TestLogisticRegression:
             "median_absolute_error",
             "mean_absolute_error",
             "mean_squared_error",
+            "root_mean_squared_error",
             "r2",
+            "r2_adj",
+            "aic",
+            "bic",
         ]
         assert reg_rep["value"][0] == pytest.approx(0.219816, abs=1e-6)
         assert reg_rep["value"][1] == pytest.approx(3.592465, abs=1e-6)
         assert reg_rep["value"][2] == pytest.approx(0.496031, abs=1e-6)
         assert reg_rep["value"][3] == pytest.approx(0.609075, abs=1e-6)
         assert reg_rep["value"][4] == pytest.approx(0.594856, abs=1e-6)
-        assert reg_rep["value"][5] == pytest.approx(0.219816, abs=1e-6)
+        assert reg_rep["value"][5] == pytest.approx(0.7712695123858948, abs=1e-6)
+        assert reg_rep["value"][6] == pytest.approx(0.219816, abs=1e-6)
+        assert reg_rep["value"][7] == pytest.approx(0.21945605202370688, abs=1e-6)
+        assert reg_rep["value"][8] == pytest.approx(-3366.7617912479104, abs=1e-6)
+        assert reg_rep["value"][9] == pytest.approx(-3339.6515694338464, abs=1e-6)
+
+        reg_rep_details = model.regression_report("details")
+        assert reg_rep_details["value"][2:] == [
+            6497.0,
+            3,
+            pytest.approx(0.219816524906085),
+            pytest.approx(0.21945605202370688),
+            pytest.approx(609.8004472783862),
+            pytest.approx(0.0),
+            pytest.approx(0.232322269343305),
+            pytest.approx(0.189622693372695),
+            pytest.approx(53.1115447611131),
+        ]
+
+        reg_rep_anova = model.regression_report("anova")
+        assert reg_rep_anova["SS"] == [
+            pytest.approx(1088.90197629059),
+            pytest.approx(3864.78372480164),
+            pytest.approx(4953.68570109281),
+        ]
+        assert reg_rep_anova["MS"][:-1] == [
+            pytest.approx(362.9673254301967),
+            pytest.approx(0.5952231210228923),
+        ]
 
     def test_score(self, model):
         # method = "max"
@@ -187,41 +267,35 @@ class TestLogisticRegression:
         # method = "median"
         assert model.score(method="median") == pytest.approx(0.496031, abs=1e-6)
         # method = "mse"
-        assert model.score(method="mse") == pytest.approx(0.609075, abs=1e-6)
+        assert model.score(method="mse") == pytest.approx(0.594856660735976, abs=1e-6)
+        # method = "rmse"
+        assert model.score(method="rmse") == pytest.approx(0.7712695123858948, abs=1e-6)
         # method = "msl"
         assert model.score(method="msle") == pytest.approx(0.002509, abs=1e-6)
         # method = "r2"
         assert model.score() == pytest.approx(0.219816, abs=1e-6)
+        # method = "r2a"
+        assert model.score(method="r2a") == pytest.approx(0.21945605202370688, abs=1e-6)
         # method = "var"
         assert model.score(method="var") == pytest.approx(0.219816, abs=1e-6)
-
-    def test_set_cursor(self, base):
-        model_test = LinearRegression("linear_reg_cursor_test", cursor=base.cursor)
-        # TODO: creat a new cursor
-        model_test.set_cursor(base.cursor)
-        model_test.drop()
-        model_test.fit("public.winequality", ["alcohol"], "quality")
-
-        base.cursor.execute(
-            "SELECT model_name FROM models WHERE model_name = 'linear_reg_cursor_test'"
-        )
-        assert base.cursor.fetchone()[0] == "linear_reg_cursor_test"
-
-        model_test.drop()
+        # method = "aic"
+        assert model.score(method="aic") == pytest.approx(-3366.7617912479104, abs=1e-6)
+        # method = "bic"
+        assert model.score(method="bic") == pytest.approx(-3339.6515694338464, abs=1e-6)
 
     def test_set_params(self, model):
         model.set_params({"max_iter": 1000})
 
         assert model.get_params()["max_iter"] == 1000
 
-    def test_model_from_vDF(self, base, winequality_vd):
-        base.cursor.execute("DROP MODEL IF EXISTS linreg_from_vDF")
-        model_test = LinearRegression("linreg_from_vDF", cursor=base.cursor)
+    def test_model_from_vDF(self, winequality_vd):
+        current_cursor().execute("DROP MODEL IF EXISTS linreg_from_vDF")
+        model_test = LinearRegression("linreg_from_vDF",)
         model_test.fit(winequality_vd, ["alcohol"], "quality")
 
-        base.cursor.execute(
+        current_cursor().execute(
             "SELECT model_name FROM models WHERE model_name = 'linreg_from_vDF'"
         )
-        assert base.cursor.fetchone()[0] == "linreg_from_vDF"
+        assert current_cursor().fetchone()[0] == "linreg_from_vDF"
 
         model_test.drop()
